@@ -51,7 +51,7 @@ main
 -- ------------------------------------------------------------
 
 exitProg        :: Bool -> IO a
-exitProg True   = exitWith (ExitFailure (-1))
+exitProg True   = exitWith (ExitFailure 1)
 exitProg False  = exitWith ExitSuccess
 
 -- ------------------------------------------------------------
