@@ -15,7 +15,7 @@ of arbitray data as XML documents
 
 This module is an adaptation of the pickler combinators
 developed by Andrew Kennedy
-( http:\/\/research.microsoft.com\/~akenn\/fun\/picklercombinators.pdf )
+( https:\/\/www.microsoft.com\/en-us\/research\/wp-content\/uploads\/2004\/01\/picklercombinators.pdf )
 
 The difference to Kennedys approach is that the target is not
 a list of Chars but a list of XmlTrees. The basic picklers will
